#include <ell/CS_LP_PivotM_Struct.hh>
#include <fstream>	// for readBinaryInputClone(..), writeBinaryOutput(..)

namespace ell
{
	CS_LP_PivotM_Struct::CS_LP_PivotM_Struct(
	 const biu::Structure& struc) {
	cStructure = compressSeq(struc,
							 S_LP_PivotM_Struct::
							  getCommonLatticeModel()->getDescriptor()
							  ->getAlphabet());
	}

	CS_LP_PivotM_Struct::
	~CS_LP_PivotM_Struct() {
	}

	std::string
	CS_LP_PivotM_Struct::getUncompressedSeqStr() const {
		return S_LP_PivotM_Struct::getCommonSequenceString();
	}

	std::string
	CS_LP_PivotM_Struct::getUncompressedStructStr() const {
			// structure string holds one character less than sequence string
		return uncompressSeq(cStructure,
							  S_LP_PivotM_Struct::
							   getCommonLatticeModel()->getDescriptor()
							   ->getAlphabet(),
							  S_LP_PivotM_Struct::
							   getCommonSequenceString().size()-1);
	}

	bool	
	CS_LP_PivotM_Struct::operator== (const CState& _cState2) const {
		CS_LP_PivotM_Struct cState2 =
		 dynamic_cast<const CS_LP_PivotM_Struct&>(_cState2);
		return (this->cStructure == cState2.cStructure);
	}

	bool
	CS_LP_PivotM_Struct::operator!= (const CState& _cState2) const {
		CS_LP_PivotM_Struct cState2 =
		 dynamic_cast<const CS_LP_PivotM_Struct&>(_cState2);
		return (this->cStructure != cState2.cStructure);
	}

	CS_LP_PivotM_Struct* 
	CS_LP_PivotM_Struct::clone() const {
		return new CS_LP_PivotM_Struct(*this);
	}

	std::string
	CS_LP_PivotM_Struct::getCStateStringRepresentation() const {
		std::string strRepr;
		for (size_t i = 0; i < cStructure.size(); i++) {
			strRepr += cStructure[i];
		}
		return strRepr;
	}

	std::string
	CS_LP_PivotM_Struct::getUncomprStateStringRepresentation()
	 const {
		return getUncompressedStructStr();
	}

	State*
	CS_LP_PivotM_Struct::uncompressedClone() const {
		return new S_LP_PivotM_Struct(
					 getUncompressedStructStr(), false);
	}

	void
	CS_LP_PivotM_Struct::uncompress(State& state) const {
		dynamic_cast<S_LP_PivotM_Struct&>(state) =
		 S_LP_PivotM_Struct(getUncompressedStructStr(), false);
	}

	CS_LP_PivotM_Struct*
	CS_LP_PivotM_Struct::readBinaryInputClone(std::ifstream& source)
	 const {

			// create new CS_LP_PivotM_Struct
		CS_LP_PivotM_Struct* cState =
		 new CS_LP_PivotM_Struct();

		size_t size;

			// read cStructure
		source.read(reinterpret_cast<char*>(&size), sizeof(size));
		cState->cStructure.resize(size);
		for (size_t i = 0; i < cState->cStructure.size(); i++) {
			source.read(reinterpret_cast<char*>(&(cState->cStructure[i])),
						  sizeof(cState->cStructure[i]));
		}

		return cState;
	}

	void
	CS_LP_PivotM_Struct::writeBinaryOutput(std::ofstream& destination)
	 const {
			// write cStructure
		size_t size = cStructure.size();
		destination.write(reinterpret_cast<const char*>(&size), sizeof(size));
		for (size_t i = 0; i < size; i++) {
			destination.write(reinterpret_cast<const char*>(&(cStructure[i])),
							   sizeof(cStructure[i]));
		}
	}

} // namespace ell
