#ifndef CLATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_
#define CLATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_


#include "CS_LP_PivotM.hh"
#include "S_LP_PivotM_StructCUBNorm.hh"

namespace ell
{	
		/**
		 * An object of this class represents a lattice protein pivot move
		 * state with a common sequence, contact energy function and
		 * a cubic lattice in a compressed representation.
		 * The structure is represented as a normalized relative move string.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CS_LP_PivotM_StructCUBNorm :
	 public CS_LP_PivotM
	{
	protected:
		CS_LP_PivotM_StructCUBNorm()
		{}

			/*! Returns the uncompressed common lattice protein sequence
			 *  string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed lattice protein structure string
			 *  (normalized relative moves).
			 */
		virtual std::string getUncompressedStructStr() const;

	public:
		CS_LP_PivotM_StructCUBNorm(
		 const biu::Structure& normStruc);

		virtual ~CS_LP_PivotM_StructCUBNorm();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CS_LP_PivotM_StructCUBNorm*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  NORMALIZEDRELATIVEMOVESTRING */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CS_LP_PivotM_StructCUBNorm*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CLATTICEPROTEINPIVOTMOVESTRUCTCUBNORMSTATE_HH_*/
