#ifndef CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_
#define CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_


#include "CS_RNAfe_SingleM.hh"
#include "S_RNAfe_SingleM_Struct.hh"

namespace ell
{	
		/**
		 * An object of this class represents a RNA free energy single move
		 * state with a common RNA sequence in a compressed representation.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class CS_RNAfe_SingleM_Struct
	 : public CS_RNAfe_SingleM
	{
	protected:
			/*! Returns the uncompressed common RNA sequence string.
			 */
		virtual std::string getUncompressedSeqStr() const;

			/*! Returns the uncompressed RNA structure string.
			 */
		virtual std::string getUncompressedStructStr() const;

		CS_RNAfe_SingleM_Struct()
		{}

	public:
		CS_RNAfe_SingleM_Struct(const biu::Structure& struc);

		virtual ~CS_RNAfe_SingleM_Struct();

	// abstract functions (CState)
		virtual bool			operator== (const CState& cState2) const;
		virtual bool			operator!= (const CState& cState2) const;

		virtual CS_RNAfe_SingleM_Struct*	clone() const;

		virtual std::string	getCStateStringRepresentation() const;

			/*! Returns a specific std::string representation of the
			 *  uncompressed representation of this state.
			 * 
			 *  BRACKETNOTATION */
		virtual std::string	getUncomprStateStringRepresentation() const;

		virtual State* uncompressedClone() const;

		virtual void	uncompress(State& state) const;

		virtual CS_RNAfe_SingleM_Struct*
				  readBinaryInputClone(std::ifstream& source) const;

		virtual void	writeBinaryOutput(std::ofstream& destination) const;
	};

}

#endif /*CRNAFREEENERGYSINGLEMOVESTRUCTSTATE_HH_*/
