#include <ell/CState.hh>
#include <cmath>	// for compress(..) and uncompress(..)
#include <biu/assertbiu.hh>

namespace ell
{

	std::vector<unsigned char>
	CState::compressSeq(const biu::Alphabet::Sequence& s, const biu::Alphabet* alph)
	 const {
			// base of the number system used for the compression
		size_t base = alph->getAlphabetSize();
			// the number of positions that can be saved in a byte
		size_t numOfPos = (size_t) (std::log(256.) / std::log((double) base));
			// ceil (s->size() / numOfPos) bytes are necessary to save the
			// sequence
		std::vector<unsigned char> cs(
		 (size_t) std::ceil((double) s.size() / (double) numOfPos), 0);

			// 1 byte = s[0] * base^0 + ... + s[numOfPos] * base^numOfPos
		for (size_t i = 0; i < s.size(); i++) {
			cs[i/numOfPos] +=
			 alph->getIndex(s.at(i)) * (int) pow(base, i%numOfPos);
		}
		return cs;
	}

	std::string
	CState::uncompressSeq(const std::vector<unsigned char>& cs,
	 const biu::Alphabet* alph, size_t seqLength) const {
			// base of the number system used for the compression
		size_t base = alph->getAlphabetSize();
			// the number of positions that can be saved in a byte
		size_t numOfPos = (size_t) (std::log(256.) / std::log((double) base));
		std::string uncompressedSeq = "";
			// element of cs which is currently processed
		unsigned char c = cs[0];

		assertbiu(seqLength <= cs.size()*numOfPos,
		 "The compressed sequence can't hold seqLength many elements.");

			// 1 byte = s[0] * base^0 + ... + s[numOfPos-1] * base^(numOfPos-1)
		for (size_t i = 0; i < seqLength; i++) {
				// update c if new element of cs is processed
			if (i%numOfPos == 0) {
				c = cs[i/numOfPos];
			}
			uncompressedSeq += alph->getString(alph->getElement(
								c % base));
			c /= base;
		}

		return uncompressedSeq;
	}

} // namespace ell
