#ifndef NEIGHBORGENERATOR_HH_
#define NEIGHBORGENERATOR_HH_


#include "State.hh"

namespace ell {
	/*!
	 * This abstract class is used to generate a list of neighbors to a given
	 * state. It's two implementations NG_Random and NG_Successive return a state's
	 * neighbors in random and successive order. 
	 * 
	 * @author Daniel Maticzka
	 */
	class NeighborGenerator
	{
	public:
	
		virtual ~NeighborGenerator(){};

		/*!
		 * Returns a NeighborListPtr to a list of neighbors to State s.
		 */
		virtual State::NeighborListPtr getNeighborList(State* s) = 0;
	};
	
	/*!
	 * This subclass of NeighborGenerator generates a randomized list of
	 * Neighbors to a given State.
	 * 
	 * @author Daniel Maticzka
	 */
	class NG_Random : public NeighborGenerator
	{
	public:

		/*!
		 * Returns a NeighborListPtr to a randomized list of neighbors to State s.
		 */
		 State::NeighborListPtr getNeighborList(State* s);
	};

	/*!
	 * This subclass of NeighborGenerator generates a list of
	 * Neighbors to a given State.
	 * 
	 * @author Daniel Maticzka
	 */
	class NG_Successive : public NeighborGenerator
	{
	public:

		/*!
		 * Returns a NeighborListPtr to a list of neighbors to State s.
		 */
		 State::NeighborListPtr getNeighborList(State* s);
	};
	
} // namespace ell

#endif /*NEIGHBORGENERATOR_HH_*/
