#include <ell/S_RNAfe_SingleM_Struct.hh>
#include <ell/CS_RNAfe_SingleM_Struct.hh>
#include <biu/assertbiu.hh>

namespace ell
{

	std::string	S_RNAfe_SingleM_Struct::commonRnaSeqStr;
//////////////////////////////////////////////////////////
// S_RNAfe_SingleM_Struct
//////////////////////////////////////////////////////////

	S_RNAfe_SingleM_Struct::S_RNAfe_SingleM_Struct(
	  const std::string& rnaStructBracketNotStr)
	 : S_RNAfe_SingleM(commonRnaSeqStr, rnaStructBracketNotStr)
	{}

	S_RNAfe_SingleM_Struct::S_RNAfe_SingleM_Struct(
	 const S_RNAfe_SingleM_Struct& rnaFreeEnStruct)
	 : S_RNAfe_SingleM(rnaFreeEnStruct) 
	{}

	S_RNAfe_SingleM_Struct::~S_RNAfe_SingleM_Struct()	{
	}

	void
	S_RNAfe_SingleM_Struct::setCommonSequenceString(
	 const std::string& seqStr) {
		assertbiu ( commonRnaSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonRnaSeqStr = seqStr;
	}

	S_RNAfe_SingleM_Struct* 
	S_RNAfe_SingleM_Struct::clone() const {
		return new S_RNAfe_SingleM_Struct(*this);
	}
	
	CState*
	S_RNAfe_SingleM_Struct::compressedClone() const {
		return new CS_RNAfe_SingleM_Struct(
		 rnaFreeEnergy.getStructure());
	}

	void
	S_RNAfe_SingleM_Struct::compress(CState& cState) const {
		dynamic_cast<CS_RNAfe_SingleM_Struct&>(cState)
		 = CS_RNAfe_SingleM_Struct(
			rnaFreeEnergy.getStructure());
	}

	State::NeighborListPtr 
	S_RNAfe_SingleM_Struct::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	S_RNAfe_SingleM_Struct::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	S_RNAfe_SingleM_Struct::NeighborList::NeighborList(
	 			const S_RNAfe_SingleM_Struct* _origin) 
	 : S_RNAfe_SingleM::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	S_RNAfe_SingleM_Struct::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;
		
		S_RNAfe_SingleM_Struct* rnaState = NULL;

		SingleMove firstMove = origin->firstValidSingleMove();

		if (firstMove != SingleMove(0,biu::RNAStructure::INVALID_INDEX)) {
			rnaState = new S_RNAfe_SingleM_Struct(*origin);
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(firstMove);
			myitstate->lastMove = firstMove;
		}
		return rnaState;
	}

	State* 
	S_RNAfe_SingleM_Struct::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

		// find next move
		SingleMove nextMove = origin->nextValidSingleMove(myitstate->lastMove);
		// no more moves possible
		if (nextMove == SingleMove(0,biu::RNAStructure::INVALID_INDEX)) {
			return NULL;
		}
		else {
			S_RNAfe_SingleM_Struct* rnaState =
			 dynamic_cast<S_RNAfe_SingleM_Struct*>(elem);
			// undo last move
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(myitstate->lastMove);
			// perform next move
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(nextMove);
			myitstate->lastMove = nextMove;
			return rnaState;
		}
	}

	// RandomNeighborList
	S_RNAfe_SingleM_Struct::RandomNeighborList::RandomNeighborList(
	 		const S_RNAfe_SingleM_Struct* _origin)
	 : S_RNAfe_SingleM::RandomNeighborList(_origin),
	   origin(_origin)
	{
	}

	State* 
	S_RNAfe_SingleM_Struct::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const 
	{
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		S_RNAfe_SingleM_Struct* rnaState = 
		 new S_RNAfe_SingleM_Struct(*origin);

		return next(*itstate, rnaState);
	}

	State* 
	S_RNAfe_SingleM_Struct::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const 
	{
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		S_RNAfe_SingleM_Struct* rnaState =
		 dynamic_cast<S_RNAfe_SingleM_Struct*>(elem);

		if (myitstate->switchModeValue > 0) {
			// positions the move will be applied to
			size_t pos1, pos2;
			SingleMove nextMove;
			bool foundValidMove = false;

			while (!foundValidMove) {
				pos1 = random() % origin->rnaFreeEnergy.getLength();
				pos2 = random() % origin->rnaFreeEnergy.getLength();
				if (pos1 == pos2) {
					continue;
				}
				nextMove = (pos1 < pos2) ? SingleMove(pos1, pos2) : SingleMove(pos2, pos1);

				if (origin->rnaFreeEnergy.isValidSingleMove(nextMove)) {
					if (!(myitstate->chosenMoves.insert(nextMove).second)) {
						// move has already been chosen --> try next
						myitstate->switchModeValue--;
						if (myitstate->switchModeValue == 0) {
							switchMode(myitstate);
							break;
						}
					}
						// move hasn't been chosen yet and is valid
						// set rnaState to the state of origin of the neighbors
					rnaState->rnaFreeEnergy = origin->rnaFreeEnergy;
					rnaState->rnaFreeEnergy.applySingleMoveInPlace(
					 nextMove);
					foundValidMove = true;
				}	
			}
		}
		if (myitstate->switchModeValue == 0) {
			size_t n = myitstate->unchosenValidMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {
				size_t i = random() % n;
					// set rnaState to the state of origin of the neighbors
				rnaState->rnaFreeEnergy = origin->rnaFreeEnergy;
				rnaState->rnaFreeEnergy.applySingleMoveInPlace(
				 myitstate->unchosenValidMoves[i]);
				myitstate->unchosenValidMoves[i] =
				 myitstate->unchosenValidMoves[n-1];
				myitstate->unchosenValidMoves.resize(n-1);
			}
		}
		return rnaState;
	}

	
	State*	
	S_RNAfe_SingleM_Struct::getRandomNeighbor(State* inPlaceNeigh) const 
	{
		S_RNAfe_SingleM_Struct* neigh = NULL;
			// copy this state
		if (inPlaceNeigh != NULL) {
			neigh = dynamic_cast<S_RNAfe_SingleM_Struct*>(inPlaceNeigh);
			//assertbiu(neigh != NULL, "inPlaceNeigh is no S_RNAfe_SingleM"); 
			//(*neigh) = (*this); 
		} else {
			neigh = new S_RNAfe_SingleM_Struct(*this);
		}
		
		size_t pos1, pos2;
		SingleMove nextMove;

		while (true) {
			pos1 = random() % neigh->rnaFreeEnergy.getLength();
			pos2 = random() % neigh->rnaFreeEnergy.getLength();
			if (pos1 == pos2) {
				continue;
			}
			nextMove = (pos1 < pos2) ? SingleMove(pos1, pos2) : SingleMove(pos2, pos1);
			// move has already been chosen
			if (rnaFreeEnergy.isValidSingleMove(nextMove)) {
					// set rnaState to the state of origin of the neighbors
				neigh->rnaFreeEnergy.applySingleMoveInPlace( nextMove );
				break;
			}	
		}
		return neigh;
	}

} // namespace ell
