#include "StateAcceptor.hh"
#include "stdlib.h" // srand & rand for SA_Metropolis
#include "math.h" // exp for SA_Metropolis


using namespace ell;

bool
SA_All::accept(WalkCollector* wc, const State& succ)
{
	return true;
}

bool
SA_E_Eq::accept(WalkCollector* wc, const State& succ)
{
	if ( succ.getEnergy() == wc->getLastState()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_Gr::accept(WalkCollector* wc, const State& succ)
{
	if ( succ.getEnergy() > wc->getLastState()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_GrEq::accept(WalkCollector* wc, const State& succ)
{
	if ( succ.getEnergy() >= wc->getLastState()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_Le::accept(WalkCollector* wc, const State& succ)
{
	if ( succ.getEnergy() < wc->getLastState()->getEnergy())
		return true;
	else
		return false;
}

bool
SA_E_LeEq::accept(WalkCollector* wc, const State& succ)
{
	if ( succ.getEnergy() <= wc->getLastState()->getEnergy())
		return true;
	else
		return false;
}

SA_Metropolis::SA_Metropolis(double _beta) :beta(_beta) {}

bool
SA_Metropolis::accept(WalkCollector* wc, const State& succ)
{	
	// compute energy difference
	double deltaE = succ.getEnergy() - wc->getLastState()->getEnergy();
	// compute probability for accepting state -> Metropolis
	double p = exp((-1)*beta*deltaE);
	
	// compute random number 0 <= r <= 1
	double r = (((double) rand()) / ((double) RAND_MAX)); 

	// return true with probability p
	if (p>r) return true;
	else return false;
}
