#ifndef WC_LISTING_HH_
#define WC_LISTING_HH_

#include "WalkCollector.hh"
#include <vector>

namespace ell 
{
	
	/*!
	 * This subclass of of WalkCollector keeps a list of all accepted States.
	 * 
	 * @author Daniel Maticzka
	 */
	class WC_Listing : public WalkCollector
	{
	private:
	
		std::vector<State*> vec;
		
	public:
	
		WC_Listing();
		~WC_Listing();

		/*!
		 * This function is used to track all accepted intermediate States
		 * visited by ell::localMin search.
		 */
		virtual void setAccepted(const State& s);
		
		/*!
		 * This function is used to track all States that are visited 
		 * but not accepted by ell::localMin search.
		 */
		virtual void setChecked(const State& s);
		
		/*!
		 * Returns last State accepted by ell::localMin search.
		 */
		virtual State* getLastState() const;
		
		/*!
		 * Returns number of accepted States.
		 */
		virtual int getAcceptedCount() const;
	
		/*!
		 * Returns a list of all accepted States.
		 */
		std::vector<State*> getAcceptedList() const;
	};
} // namespace ell

#endif /*WC_LISTING_HH_*/
