#ifndef WALKCOLLECTOR_HH_
#define WALKCOLLECTOR_HH_

#include "State.hh"
#include <vector>

namespace ell 
{
	/*!
	 * This class is used by the ell::localMin function to keep track
	 * of visited States.
	 * 
	 * @author Daniel Maticzka
	 */
	class WalkCollector
	{
	private:
	
		State* lastState;
		int count;
		
	public:	
	
		WalkCollector();
		
		virtual
		~WalkCollector();
	
		/*!
		 * This function is used to track all accepted intermediate States
		 * visited by ell::localMin search.
		 */
		virtual void setAccepted(const State& s);
		
		/*!
		 * This function is used to track all States that are visited 
		 * but not accepted by ell::localMin search.
		 */
		virtual void setChecked(const State& s);
		
		/*!
		 * Returns last State accepted by ell::localMin search.
		 */
		virtual State* getLastState() const;
		
		/*!
		 * Returns number of accepted States.
		 */
		 virtual int getAcceptedCount() const;
	};
	
} // namespace ell

#endif /*WALKCOLLECTOR_HH_*/
