#include "NeighborGenerator.hh"
#include "State.hh"
#include <iostream>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id in the constructor and retrieving it via
 * getStateStringRepresentation.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;

	DummyState(std::string _id)
	{
		id = _id; 
	}
	
	~DummyState()
	{}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return 0;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	State* clone() const
	{
		return new DummyState(id);
	}
	
	std::string	getStateStringRepresentation() const 
	{
		return id;
	}

	CState*	compressedClone() const
	{
		return NULL;
	}

	void compress(CState& cState) const {}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}
};

int main(int argc, char** argv)
{
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : NeighborGenerator\n"
				<<"===========================================" <<std::endl;
	
	std::cout 	<< "=> creating DummyState" << std::endl;
	State* s = new DummyState("dummy");
	
	std::cout 	<< "=> creating NG_Random" << std::endl;
	NeighborGenerator* ng1 = new NG_Random();
	std::cout 	<< "->getNeighborList(s)" << std::endl;
	ng1->getNeighborList(s);
	delete ng1;

	std::cout 	<< "=> creating NG_Successive" << std::endl;
	NeighborGenerator* ng2 = new NG_Successive();
	std::cout 	<< "->getNeighborList(s)" << std::endl;
	ng2->getNeighborList(s);
	delete ng2;
	
	delete s;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
