#include <iostream>
#include <biu/LatticeDescriptorCUB.hh>
#include <ell/S_LP_PivotM.hh>
#include <time.h>
#include <fstream>

int main(int argc, char **argv) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : S_LP_PivotM\n"
				<<"===========================================" <<std::endl;
				
		// erzeugen	
	srandom( (unsigned) time(NULL) );

	std::cout	<<"=> creating biu::LatticeDescriptorCUB() = ld" <<std::endl;
		biu::LatticeDescriptorCUB ld;
	std::cout	<<"=> creating biu::LatticeModel(ld) = lm" <<std::endl;
		biu::LatticeModel lm(&ld);
	std::cout	<<"=> creating biu::Alphabet(HP,1) = alph" <<std::endl;
		biu::Alphabet alph("HP",1);
	std::cout	<<"=> creating biu::EnergyMatrix(0.0,2,2) = E" <<std::endl;
		biu::EnergyMatrix E = biu::EnergyMatrix(	0.0, 
									alph.getAlphabetSize(),
									alph.getAlphabetSize());
		E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	std::cout	<<E <<std::endl;
	std::cout	<<"=> creating biu::ContactEnergyFunction( alph, E) = f" 
				<<std::endl;
		biu::ContactEnergyFunction f(&alph, &E);
		std::string seqStr = "PHPHHPH";
	std::cout	<<"=> "<<seqStr <<" = seqStr" <<std::endl;
		std::string relMoveStr = "FRUULU";
	std::cout	<<"=> "<<relMoveStr <<"  = relMoveStr" <<std::endl;
		std::string absMoveStr = lm.getString(
									lm.relMovesToAbsMoves(
										lm.getDescriptor()->getSequence(
											relMoveStr)));
	std::cout	<<"=> "<<absMoveStr<<"  = absMoveStr" <<std::endl;

	std::cout	<<"=> creating ell::S_LP_PivotM("
				<<"lm,f,seqStr,relMoveStr,false) = lp" 
				<<std::endl;
		S_LP_PivotM lp(&lm,&f,seqStr,relMoveStr,false);

		// getStateStringRepresentation()
	std::cout	<<"->getStateStringRepresentation()  = "
				<<lp.getStateStringRepresentation()
				<<std::endl;

		// getFitness()
	std::cout	<<"->getFitness()  = "
				<<(lp.getFitness())
				<<std::endl;

	ell::State* lp2 = lp.clone();

		// clone()  ==
	std::cout	<<"->(*this == *(this->clone()))  = "
				<<(lp == *lp2 ? "true":"false")
				<<std::endl;

		// clone()  !=
	std::cout	<<"->(*this != *(this->clone()))  = "
				<<(lp != *lp2 ? "true":"false")
				<<std::endl;

	int num = 0;
	
		// getNeighborList()
	ell::State::NeighborListPtr nl = lp.getNeighborList();
	std::set<std::string> neighs, randneighs;
	std::set<std::string>::const_iterator setIt;
	std::cout	<<"->getNeighborList()  ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		neighs.insert(it->getStateStringRepresentation());
	}
	for(setIt = neighs.begin(); setIt != neighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}


		// getRandomNeighborList()
	num = 0;
	nl = lp.getRandomNeighborList();
	std::cout	<<"->getRandomNeighborList() ==> now sorted" <<std::endl;
	for(ell::State::NeighborList::Iterator it = nl->begin();
		 it != nl->end(); ++it) {
		randneighs.insert(it->getStateStringRepresentation());
	}
	for(setIt = randneighs.begin(); setIt != randneighs.end(); setIt++) {
		std::cout <<" " <<++num <<"\t: " <<*setIt <<std::endl;
	}
	
		// getMinimalDistance(..)
	std::cout	<<"Distance to PHPHHPH FRDDLD: "
				<<lp.getMinimalDistance(
					ell::S_LP_PivotM(&lm,&f,"PHPHHPH",
														"FRDDLD",false))
				<<std::endl;

	ell::CState* cLp = lp.compressedClone();
	ell::CState* cLp2 = lp2->compressedClone();
	lp.compress(*cLp2);

		// compressedClone()  ==   compress(..)
	std::cout	<<"=>this.compress(cLp)" <<std::endl;
	std::cout	<<"->(*(this.compressedClone()) == cLp)  = "
				<<(*cLp == *cLp2 ? "true":"false")
				<<std::endl;

		// compressedClone()  !=   compress(..)
	std::cout	<<"->(*(this.compressedClone()) != cLp)  = "
				<<(*cLp != *cLp2 ? "true":"false")
				<<std::endl;

	ell::CState* cLp4 = cLp2->clone();

		// cLp  ==  *(cLp.clone())
	std::cout	<<"->(cLp == *(cLp.clone()))  = "
				<<(*cLp2 == *cLp4 ? "true":"false")
				<<std::endl;

		// getCStateStringRepresentation()
	std::cout	<<"->(this.compressedClone()->getCStateStringRepresentation()"
				<<"  = "
				<<cLp->getCStateStringRepresentation()
				<<std::endl;

		// getUncomprStateStringRepresentation()
	std::cout	<<"->getUncomprStateStringRepresentation()  = "
				<<cLp->getUncomprStateStringRepresentation()
				<<std::endl;

	ell::State* lp3 = cLp->uncompressedClone();

		// uncompressedClone()
	std::cout	<<"->(*(this.compressedClone()->uncompressedClone()) == "
				<<"*(this))  = "
				<<(*lp3 == lp ? "true":"false")
				<<std::endl;

	cLp->uncompress(*lp2);

		// uncompress(..)
	std::cout	<<"=>this.compressedClone()->uncompress(lp)" <<std::endl;
	std::cout	<<"->(*(this) == lp)  = "
				<<(*lp2 == lp ? "true":"false")
				<<std::endl;

	std::ofstream out("out.bin");
	if (!out) {
		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
		exit(-1);
	}
	cLp2->writeBinaryOutput(out);
	out.close();
	std::ifstream in("out.bin");
	if (!in) {
		std::cerr<<"out.bin couldn't be opened!"<<std::endl;
		exit(-1);
	}
	ell::CState* cLp3 = cLp2->readBinaryInputClone(in);
	in.close();

		// writeBinaryOutput(..), readBinaryInputClone(..)
	std::cout	<<"=>std::ofstream out(\"out.bin\")" <<std::endl;
	std::cout	<<"=>cRna.writeBinaryOutput(out)" <<std::endl;
	std::cout	<<"=>std::ifstream in(\"out.bin\")" <<std::endl;
	std::cout	<<"=>cLp2 = cLp.readBinaryInputClone(in)" <<std::endl;
	std::cout	<<"->(*cLp == *cLp2)  = "
				<<(*cLp2 == *cLp3 ? "true":"false")
				<<std::endl;

	delete lp2;
	delete lp3;
	delete cLp;
	delete cLp2;
	delete cLp3;
	delete cLp4;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
