#include "WalkAbortionCriterion.hh"
#include "WalkCollector.hh"
#include "State.hh"
#include <iostream>

using namespace ell;

/*!
 * This is a dummy implementation of a State class. The only functionality
 * implemented is setting an id and energy in the constructor and retrieving it via
 * getStateStringRepresentation and getEnergy.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
	public:
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy)
	{
		id = _id; 
		energy = _energy;
	}
	
	~DummyState()
	{}
	
	bool operator== (const State& state2) const {return true;}
	bool operator!= (const State& state2) const {return true;}

	int	getFitness() const 
	{
		return 0;
	}
	
	double getEnergy() const 
	{
		return energy;
	}
	
	unsigned int getMinimalDistance(const State& state2) const 
	{
		return 0;
	}
	
	State* clone() const
	{
		return new DummyState(id,energy);
	}
	
	std::string	getStateStringRepresentation() const 
	{
		return id;
	}

	CState*	compressedClone() const
	{
		return NULL;
	}

	void compress(CState& cState) const {}

	NeighborListPtr getNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	NeighborListPtr getRandomNeighborList() const
	{
		return *(new NeighborListPtr);
	}

	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const
	{
		return NULL;
	}
};


int main(int argc, char** argv)
{
	
	/*
	 * Test WAC_MaxEnergy
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MaxEnergy\n"
				<<"===========================================" <<std::endl;

	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	State* s = new DummyState("State 1", 5);
	std::cout << "->getEnergy = " << s->getEnergy() << std::endl;
	std::cout << "=> creating WalkCollector wc" << std::endl;
	WalkCollector* wc = new WalkCollector();
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	
	std::cout << "=> creating WAC_MaxEnergy(4)" << std::endl;
	WalkAbortionCriterion* wacMaxEnergy = new WAC_MaxEnergy(4);
	std::cout << "->abort(wc) = " << wacMaxEnergy->abort(wc) << std::endl;
	
	std::cout << "=> creating WAC_MaxEnergy(6)" << std::endl;
	delete wacMaxEnergy;
	wacMaxEnergy = new WAC_MaxEnergy(6);
	std::cout << "->abort(wc) = " << wacMaxEnergy->abort(wc) << std::endl;
	
	delete s;
	delete wc;
	delete wacMaxEnergy;
	
	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	
	/*
	 * Test WAC_MinEnergy
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MinEnergy\n"
				<<"===========================================" <<std::endl;
	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	s = new DummyState("State 1", 5);
	std::cout << "->getEnergy = " << s->getEnergy() << std::endl;
	std::cout << "=> creating WalkCollector wc" << std::endl;
	wc = new WalkCollector();
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	
	std::cout << "=> creating WAC_MinEnergy(4)" << std::endl;
	WalkAbortionCriterion* wacMinEnergy = new WAC_MinEnergy(4);
	std::cout << "->abort(wc) = " << wacMinEnergy->abort(wc) << std::endl;
		
	std::cout << "=> creating WAC_MinEnergy(6)" << std::endl;
	delete wacMinEnergy;
	wacMinEnergy = new WAC_MinEnergy(6);
	std::cout << "->abort(wc) = " << wacMinEnergy->abort(wc) << std::endl;
	
	delete s;
	delete wc;
	delete wacMinEnergy;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	/*
	 * Test WAC_MaxLength
	 */
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test class : WAC_MaxLength\n"
				<<"===========================================" <<std::endl;
	std::cout << "=> creating DummyState(\"State 1\", 5)" << std::endl;
	s = new DummyState("State 1", 5);
	std::cout << "=> creating WalkCollector wc" << std::endl;
	wc = new WalkCollector();
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	std::cout << "->setAccepted(*s)" << std::endl;
	wc->setAccepted(*s);
	std::cout << "->getAcceptedCount: " << wc->getAcceptedCount() << std::endl;
	
	std::cout << "=> creating WAC_MaxLength(4)" << std::endl;
	WalkAbortionCriterion* wacMaxLength = new WAC_MaxLength(4);
	std::cout << "->abort(wc) = " << wacMaxLength->abort(wc) << std::endl;
		
	std::cout << "=> creating WAC_MaxLength(6)" << std::endl;
	delete wacMaxLength;
	wacMaxLength = new WAC_MaxLength(6);
	std::cout << "->abort(wc) = " << wacMaxLength->abort(wc) << std::endl;
	
	delete s;
	delete wc;
	delete wacMaxLength;

	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
	
	return 0;
}
