
#include "ell/walkAdaptive.hh"
#include "ell/WC_Listing.hh"
#include "ell/State.hh"
#include <iostream>
#include <string>
#include <vector>

using namespace ell;

/*!
 * This is a dummy implementation of a State class including inner classes
 * NeighborList and ItState.
 * 
 * @author Daniel Maticzka
 */
class DummyState : public State
{
public:
	class NeighborList : public State::NeighborList 
	{
	private:
		std::string id;
		double energy;
	public:
		class ItState : public State::NeighborList::ItState {
		public:
			int pos;
			
			ItState():pos(0) {}
			
			ItState(const ItState &itstate) {pos=itstate.pos;}
			
			virtual ItState * clone() const {
				return new ItState(*this);
			}
		};
    
		NeighborList(std::string _id, double _energy) 
		{
			id = _id;
			energy = _energy;
		}
    
		virtual ~NeighborList() {}
  
		virtual State*
		next(State::NeighborList::ItState *itstate, State* elem) const {
			((DummyState*)elem)->applyMove();
			if ( ((DummyState*)elem)->getEnergy()==0 ) return NULL;
			else return elem;
		}

		virtual State* 
		first(State::NeighborList::ItState** itstate) const {
			DummyState* s = new DummyState(id, energy);
			s->applyMove();
			return s;
		}
	};
	
	std::string id;
	double energy;

	DummyState(std::string _id, double _energy) { id = _id; energy = _energy; } 
	~DummyState() {}
	
	bool operator== (const State& state2) const { return true; }
	bool operator!= (const State& state2) const { return true; }

	int	getFitness() const { return 0; }
	double getEnergy() const { return energy; }
	unsigned int getMinimalDistance(const State& state2) const { return 0; }
	State* clone() const { return new DummyState(id, energy); }
	std::string	getStateStringRepresentation() const { return id; }
	CState*	compressedClone() const { return NULL; }
	void compress(CState& cState) const {}
	
	NeighborListPtr getNeighborList() const 
	{ 
		return NeighborListPtr(new NeighborList(id,energy));
	}
	
	NeighborListPtr getRandomNeighborList() const {
		return NeighborListPtr(new NeighborList(id,energy));
	}
	
	State* getRandomNeighbor(State* inPlaceNeigh = NULL) const { return NULL; }
	
	DummyState& applyMove() {
		if (energy > 0) {
			energy -= 1;
		}
		return *this;
	}
};

int main(int argc, char **argv) {
	
	std::cout	<<"========== BEGIN OF TEST ==================\n"
				<<" test function : walkAdaptive\n"
				<<"===========================================" <<std::endl;

	std::cout	<< "=> creating DummyState" << std::endl;
	State* state = new DummyState("state 1", 10);
	std::cout	<< "->getEnergy = " << state->getEnergy() << std::endl;

	std::cout	<< "=> creating WalkCollector" << std::endl;
	WalkCollector* wc = new WC_Listing();
	
	std::cout	<< "=> calling walk" << std::endl;
	wc = walkAdaptive(state, wc, RANDOM);
	
	std::cout 	<< "Number Of Accepted States: " 
					<< wc->getAcceptedCount()
					<< std::endl;
	
	WC_Listing* wch;
	wch = dynamic_cast<WC_Listing*>(wc);
	
	std::cout 	<< "Accepted States:" << std::endl;
	std::vector<State*> vec = wch->getAcceptedList();
	std::vector<State*>::iterator iter;
  	for( iter = vec.begin(); iter != vec.end(); iter++ ) {
    	std::cout 	<< "  * Energy = "
    				<< (*iter)->getEnergy() << std::endl;
  	}
	
  	delete state;
  	delete wc;
  	
  	std::cout	<<"============= END OF TEST =================\n" <<std::endl;
  	
	return 0; 
}
